/*
  Constellations.cpp
  Copyright (C) 2007 David M. Anderson

  The constellations in the sky.
*/


#include "Constellations.hpp"
#include "Precession.hpp"
#include "Array.hpp"
#include "Assert.hpp"
#ifdef DEBUG
#include <iostream>
#include "TestCheck.hpp"
#include "AngleHMS.hpp"
#endif
using namespace std;


namespace EpsilonDelta
{                                                      //namespace EpsilonDelta

//*****************************************************************************


namespace
{                                                                   //namespace
//-----------------------------------------------------------------------------

ConstellationInfo s_constellationInfo[ NumConstellations ]
= {
    ConstellationInfo( "Andromeda",         "And", "Andromedae" ),
    ConstellationInfo( "Antlia",            "Ant", "Antliae" ),
    ConstellationInfo( "Apus",              "Aps", "Apodis" ),
    ConstellationInfo( "Aquarius",          "Aqr", "Aquarii" ),
    ConstellationInfo( "Aquila",            "Aql", "Aquilae" ),
    ConstellationInfo( "Ara",               "Ara", "Arae" ),
    ConstellationInfo( "Aries",             "Ari", "Arietis" ),
    ConstellationInfo( "Auriga",            "Aur", "Aurigae" ),
    ConstellationInfo( "Boötes",            "Boo", "Boötis" ),
    ConstellationInfo( "Caelum",            "Cae", "Caeli" ),
    ConstellationInfo( "Camelopardalis",    "Cam", "Camelopardalis" ),
    ConstellationInfo( "Cancer",            "Cnc", "Cancri" ),
    ConstellationInfo( "Canes Venatici",    "CVn", "Canum Venaticorum" ),
    ConstellationInfo( "Canis Major",       "CMa", "Canis Majoris" ),
    ConstellationInfo( "Canis Minor",       "CMi", "Canis Minoris" ),
    ConstellationInfo( "Capricornus",       "Cap", "Capricorni" ),
    ConstellationInfo( "Carina",            "Car", "Carinae" ),
    ConstellationInfo( "Cassiopeia",        "Cas", "Cassiopeiae" ),
    ConstellationInfo( "Centaurus",         "Cen", "Centauri" ),
    ConstellationInfo( "Cepheus",           "Cep", "Cephei" ),
    ConstellationInfo( "Cetus",             "Cet", "Ceti" ),
    ConstellationInfo( "Chamaeleon",        "Cha", "Chamaeleontis" ),
    ConstellationInfo( "Circinus",          "Cir", "Circini" ),
    ConstellationInfo( "Columba",           "Col", "Columbae" ),
    ConstellationInfo( "Coma Berenices",    "Com", "Comae Berenices" ),
    ConstellationInfo( "Corona Australis",  "CrA", "Coronae Australis" ),
    ConstellationInfo( "Corona Borealis",   "CrB", "Coronae Borealis" ),
    ConstellationInfo( "Corvus",            "Crv", "Corvi" ),
    ConstellationInfo( "Crater",            "Crt", "Crateris" ),
    ConstellationInfo( "Crux",              "Cru", "Crucis" ),
    ConstellationInfo( "Cygnus",            "Cyg", "Cygni" ),
    ConstellationInfo( "Delphinus",         "Del", "Delphini" ),
    ConstellationInfo( "Dorado",            "Dor", "Doradus" ),
    ConstellationInfo( "Draco",             "Dra", "Draconis" ),
    ConstellationInfo( "Equuleus",          "Equ", "Equulei" ),
    ConstellationInfo( "Eridanus",          "Eri", "Eridani" ),
    ConstellationInfo( "Fornax",            "For", "Fornacis" ),
    ConstellationInfo( "Gemini",            "Gem", "Geminorum" ),
    ConstellationInfo( "Grus",              "Gru", "Gruis" ),
    ConstellationInfo( "Hercules",          "Her", "Herculis" ),
    ConstellationInfo( "Horologium",        "Hor", "Horologii" ),
    ConstellationInfo( "Hydra",             "Hya", "Hydrae" ),
    ConstellationInfo( "Hydrus",            "Hyi", "Hydri" ),
    ConstellationInfo( "Indus",             "Ind", "Indi" ),
    ConstellationInfo( "Lacerta",           "Lac", "Lacertae" ),
    ConstellationInfo( "Leo",               "Leo", "Leonis" ),
    ConstellationInfo( "Leo Minor",         "LMi", "Leonis Minoris" ),
    ConstellationInfo( "Lepus",             "Lep", "Leporis" ),
    ConstellationInfo( "Libra",             "Lib", "Librae" ),
    ConstellationInfo( "Lupus",             "Lup", "Lupi" ),
    ConstellationInfo( "Lynx",              "Lyn", "Lyncis" ),
    ConstellationInfo( "Lyra",              "Lyr", "Lyrae" ),
    ConstellationInfo( "Mensa",             "Men", "Mensae" ),
    ConstellationInfo( "Microscopium",      "Mic", "Microscopii" ),
    ConstellationInfo( "Monoceros",         "Mon", "Monocerotis" ),
    ConstellationInfo( "Musca",             "Mus", "Muscae" ),
    ConstellationInfo( "Norma",             "Nor", "Normae" ),
    ConstellationInfo( "Octans",            "Oct", "Octantis" ),
    ConstellationInfo( "Ophiuchus",         "Oph", "Ophiuchi" ),
    ConstellationInfo( "Orion",             "Ori", "Orionis" ),
    ConstellationInfo( "Pavo",              "Pav", "Pavonis" ),
    ConstellationInfo( "Pegasus",           "Peg", "Pegasi" ),
    ConstellationInfo( "Perseus",           "Per", "Persei" ),
    ConstellationInfo( "Phoenix",           "Phe", "Phoenicis" ),
    ConstellationInfo( "Pictor",            "Pic", "Pictoris" ),
    ConstellationInfo( "Pisces",            "Psc", "Piscium" ),
    ConstellationInfo( "Piscis Austrinus",  "PsA", "Piscis Austrini" ),
    ConstellationInfo( "Puppis",            "Pup", "Puppis" ),
    ConstellationInfo( "Pyxis",             "Pyx", "Pyxidis" ),
    ConstellationInfo( "Reticulum",         "Ret", "Reticuli" ),
    ConstellationInfo( "Sagitta",           "Sge", "Sagittae" ),
    ConstellationInfo( "Sagittarius",       "Sgr", "Sagittarii" ),
    ConstellationInfo( "Scorpius",          "Sco", "Scorpii" ),
    ConstellationInfo( "Sculptor",          "Scl", "Sculptoris" ),
    ConstellationInfo( "Scutum",            "Sct", "Scuti" ),
    ConstellationInfo( "Serpens",           "Ser", "Serpentis" ),
    ConstellationInfo( "Sextans",           "Sex", "Sextantis" ),
    ConstellationInfo( "Taurus",            "Tau", "Tauri" ),
    ConstellationInfo( "Telescopium",       "Tel", "Telescopii" ),
    ConstellationInfo( "Triangulum",        "Tri", "Trianguli" ),
    ConstellationInfo( "Triangulum Australe", "TrA", "Trianguli Australis" ),
    ConstellationInfo( "Tucana",            "Tuc", "Tucanae" ),
    ConstellationInfo( "Ursa Major",        "UMa", "Ursae Majoris" ),
    ConstellationInfo( "Ursa Minor",        "UMi", "Ursae Minoris" ),
    ConstellationInfo( "Vela",              "Vel", "Velorum" ),
    ConstellationInfo( "Virgo",             "Vir", "Virginis" ),
    ConstellationInfo( "Volans",            "Vol", "Volantis" ),
    ConstellationInfo( "Vulpecula",         "Vul", "Vulpeculae" )
};

//=============================================================================

/* The data and algorithm used here is based on:
   Roman N. G., "Identification of a Constellation from a Position",
   <Pub. Astron. Soc. Pac. 99, 695, (1987)> =1987PASP...99..695R
*/

struct BoundaryDatum
{
    Angle raLower;
    Angle raUpper;
    Angle decLower;
    Constellation constellation;
};

BoundaryDatum s_boundaryData[]
= {
    { Angle(   0.0000, Angle::Degree ), Angle( 360.0000, Angle::Degree ),
      Angle( 88.0000, Angle::Degree ), UrsaMinor },
    { Angle( 120.0000, Angle::Degree ), Angle( 217.5000, Angle::Degree ),
      Angle( 86.5000, Angle::Degree ), UrsaMinor },
    { Angle( 315.0000, Angle::Degree ), Angle( 345.0000, Angle::Degree ),
      Angle( 86.1667, Angle::Degree ), UrsaMinor },
    { Angle( 270.0000, Angle::Degree ), Angle( 315.0000, Angle::Degree ),
      Angle( 86.0000, Angle::Degree ), UrsaMinor },
    { Angle(   0.0000, Angle::Degree ), Angle( 120.0000, Angle::Degree ),
      Angle( 85.0000, Angle::Degree ), Cepheus },
    { Angle( 137.5000, Angle::Degree ), Angle( 160.0000, Angle::Degree ),
      Angle( 82.0000, Angle::Degree ), Camelopardalis },
    { Angle(   0.0000, Angle::Degree ), Angle(  75.0000, Angle::Degree ),
      Angle( 80.0000, Angle::Degree ), Cepheus },
    { Angle( 160.0000, Angle::Degree ), Angle( 217.5000, Angle::Degree ),
      Angle( 80.0000, Angle::Degree ), Camelopardalis },
    { Angle( 262.5000, Angle::Degree ), Angle( 270.0000, Angle::Degree ),
      Angle( 80.0000, Angle::Degree ), UrsaMinor },
    { Angle( 302.5000, Angle::Degree ), Angle( 315.0000, Angle::Degree ),
      Angle( 80.0000, Angle::Degree ), Draco },
    { Angle(   0.0000, Angle::Degree ), Angle(  52.6250, Angle::Degree ),
      Angle( 77.0000, Angle::Degree ), Cepheus },
    { Angle( 172.5000, Angle::Degree ), Angle( 203.7500, Angle::Degree ),
      Angle( 77.0000, Angle::Degree ), Camelopardalis },
    { Angle( 248.0000, Angle::Degree ), Angle( 262.5000, Angle::Degree ),
      Angle( 75.0000, Angle::Degree ), UrsaMinor },
    { Angle( 302.5000, Angle::Degree ), Angle( 310.0000, Angle::Degree ),
      Angle( 75.0000, Angle::Degree ), Cepheus },
    { Angle( 119.5000, Angle::Degree ), Angle( 137.5000, Angle::Degree ),
      Angle( 73.5000, Angle::Degree ), Camelopardalis },
    { Angle( 137.5000, Angle::Degree ), Angle( 170.0000, Angle::Degree ),
      Angle( 73.5000, Angle::Degree ), Draco },
    { Angle( 195.0000, Angle::Degree ), Angle( 248.0000, Angle::Degree ),
      Angle( 70.0000, Angle::Degree ), UrsaMinor },
    { Angle(  46.5000, Angle::Degree ), Angle(  51.2500, Angle::Degree ),
      Angle( 68.0000, Angle::Degree ), Cassiopeia },
    { Angle( 306.2500, Angle::Degree ), Angle( 310.0000, Angle::Degree ),
      Angle( 67.0000, Angle::Degree ), Draco },
    { Angle( 170.0000, Angle::Degree ), Angle( 180.0000, Angle::Degree ),
      Angle( 66.5000, Angle::Degree ), Draco },
    { Angle(   0.0000, Angle::Degree ), Angle(   5.0000, Angle::Degree ),
      Angle( 66.0000, Angle::Degree ), Cepheus },
    { Angle( 210.0000, Angle::Degree ), Angle( 235.0000, Angle::Degree ),
      Angle( 66.0000, Angle::Degree ), UrsaMinor },
    { Angle( 353.7500, Angle::Degree ), Angle( 360.0000, Angle::Degree ),
      Angle( 66.0000, Angle::Degree ), Cepheus },
    { Angle( 180.0000, Angle::Degree ), Angle( 202.5000, Angle::Degree ),
      Angle( 64.0000, Angle::Degree ), Draco },
    { Angle( 202.5000, Angle::Degree ), Angle( 216.2500, Angle::Degree ),
      Angle( 63.0000, Angle::Degree ), Draco },
    { Angle( 347.5000, Angle::Degree ), Angle( 353.7500, Angle::Degree ),
      Angle( 63.0000, Angle::Degree ), Cepheus },
    { Angle(  91.5000, Angle::Degree ), Angle( 105.0000, Angle::Degree ),
      Angle( 62.0000, Angle::Degree ), Camelopardalis },
    { Angle( 300.0000, Angle::Degree ), Angle( 306.2500, Angle::Degree ),
      Angle( 61.5000, Angle::Degree ), Draco },
    { Angle( 308.0500, Angle::Degree ), Angle( 309.0000, Angle::Degree ),
      Angle( 60.9167, Angle::Degree ), Cepheus },
    { Angle( 105.0000, Angle::Degree ), Angle( 119.5000, Angle::Degree ),
      Angle( 60.0000, Angle::Degree ), Camelopardalis },
    { Angle( 119.5000, Angle::Degree ), Angle( 126.2500, Angle::Degree ),
      Angle( 60.0000, Angle::Degree ), UrsaMajor },
    { Angle( 296.5000, Angle::Degree ), Angle( 300.0000, Angle::Degree ),
      Angle( 59.5000, Angle::Degree ), Draco },
    { Angle( 300.0000, Angle::Degree ), Angle( 308.0500, Angle::Degree ),
      Angle( 59.5000, Angle::Degree ), Cepheus },
    { Angle( 343.0000, Angle::Degree ), Angle( 347.5000, Angle::Degree ),
      Angle( 59.0833, Angle::Degree ), Cepheus },
    { Angle(   0.0000, Angle::Degree ), Angle(  36.5000, Angle::Degree ),
      Angle( 58.5000, Angle::Degree ), Cassiopeia },
    { Angle( 291.2500, Angle::Degree ), Angle( 296.5000, Angle::Degree ),
      Angle( 58.0000, Angle::Degree ), Draco },
    { Angle(  25.5000, Angle::Degree ), Angle(  28.6250, Angle::Degree ),
      Angle( 57.5000, Angle::Degree ), Cassiopeia },
    { Angle(  36.5000, Angle::Degree ), Angle(  46.5000, Angle::Degree ),
      Angle( 57.0000, Angle::Degree ), Cassiopeia },
    { Angle(  46.5000, Angle::Degree ), Angle(  47.5000, Angle::Degree ),
      Angle( 57.0000, Angle::Degree ), Camelopardalis },
    { Angle( 334.7500, Angle::Degree ), Angle( 343.0000, Angle::Degree ),
      Angle( 56.2500, Angle::Degree ), Cepheus },
    { Angle(  75.0000, Angle::Degree ), Angle(  91.5000, Angle::Degree ),
      Angle( 56.0000, Angle::Degree ), Camelopardalis },
    { Angle( 210.5000, Angle::Degree ), Angle( 216.2500, Angle::Degree ),
      Angle( 55.5000, Angle::Degree ), UrsaMajor },
    { Angle( 216.2500, Angle::Degree ), Angle( 291.2500, Angle::Degree ),
      Angle( 55.5000, Angle::Degree ), Draco },
    { Angle(  47.5000, Angle::Degree ), Angle(  50.0000, Angle::Degree ),
      Angle( 55.0000, Angle::Degree ), Camelopardalis },
    { Angle( 332.0000, Angle::Degree ), Angle( 334.7500, Angle::Degree ),
      Angle( 55.0000, Angle::Degree ), Cepheus },
    { Angle( 309.0000, Angle::Degree ), Angle( 329.5000, Angle::Degree ),
      Angle( 54.8333, Angle::Degree ), Cepheus },
    { Angle(   0.0000, Angle::Degree ), Angle(  25.5000, Angle::Degree ),
      Angle( 54.0000, Angle::Degree ), Cassiopeia },
    { Angle(  91.5000, Angle::Degree ), Angle(  97.5000, Angle::Degree ),
      Angle( 54.0000, Angle::Degree ), Lynx },
    { Angle( 181.2500, Angle::Degree ), Angle( 202.5000, Angle::Degree ),
      Angle( 53.0000, Angle::Degree ), UrsaMajor },
    { Angle( 228.7500, Angle::Degree ), Angle( 236.2500, Angle::Degree ),
      Angle( 53.0000, Angle::Degree ), Draco },
    { Angle( 329.5000, Angle::Degree ), Angle( 332.0000, Angle::Degree ),
      Angle( 52.7500, Angle::Degree ), Cepheus },
    { Angle(  50.0000, Angle::Degree ), Angle(  75.0000, Angle::Degree ),
      Angle( 52.5000, Angle::Degree ), Camelopardalis },
    { Angle( 343.0000, Angle::Degree ), Angle( 350.0000, Angle::Degree ),
      Angle( 52.5000, Angle::Degree ), Cassiopeia },
    { Angle( 236.2500, Angle::Degree ), Angle( 255.0000, Angle::Degree ),
      Angle( 51.5000, Angle::Degree ), Draco },
    { Angle(  30.6250, Angle::Degree ), Angle(  37.7500, Angle::Degree ),
      Angle( 50.5000, Angle::Degree ), Perseus },
    { Angle( 255.0000, Angle::Degree ), Angle( 273.5000, Angle::Degree ),
      Angle( 50.5000, Angle::Degree ), Draco },
    { Angle(   0.0000, Angle::Degree ), Angle(  20.5000, Angle::Degree ),
      Angle( 50.0000, Angle::Degree ), Cassiopeia },
    { Angle(  20.5000, Angle::Degree ), Angle(  25.0000, Angle::Degree ),
      Angle( 50.0000, Angle::Degree ), Perseus },
    { Angle(  97.5000, Angle::Degree ), Angle( 102.0000, Angle::Degree ),
      Angle( 50.0000, Angle::Degree ), Lynx },
    { Angle( 350.0000, Angle::Degree ), Angle( 360.0000, Angle::Degree ),
      Angle( 50.0000, Angle::Degree ), Cassiopeia },
    { Angle( 202.5000, Angle::Degree ), Angle( 210.5000, Angle::Degree ),
      Angle( 48.5000, Angle::Degree ), UrsaMajor },
    { Angle(   0.0000, Angle::Degree ), Angle(  16.7500, Angle::Degree ),
      Angle( 48.0000, Angle::Degree ), Cassiopeia },
    { Angle( 353.7500, Angle::Degree ), Angle( 360.0000, Angle::Degree ),
      Angle( 48.0000, Angle::Degree ), Cassiopeia },
    { Angle( 272.6250, Angle::Degree ), Angle( 273.5000, Angle::Degree ),
      Angle( 47.5000, Angle::Degree ), Hercules },
    { Angle( 273.5000, Angle::Degree ), Angle( 286.2500, Angle::Degree ),
      Angle( 47.5000, Angle::Degree ), Draco },
    { Angle( 286.2500, Angle::Degree ), Angle( 287.5000, Angle::Degree ),
      Angle( 47.5000, Angle::Degree ), Cygnus },
    { Angle(  25.0000, Angle::Degree ), Angle(  30.6250, Angle::Degree ),
      Angle( 47.0000, Angle::Degree ), Perseus },
    { Angle( 126.2500, Angle::Degree ), Angle( 137.5000, Angle::Degree ),
      Angle( 47.0000, Angle::Degree ), UrsaMajor },
    { Angle(   2.5000, Angle::Degree ), Angle(  13.0000, Angle::Degree ),
      Angle( 46.0000, Angle::Degree ), Cassiopeia },
    { Angle( 180.0000, Angle::Degree ), Angle( 181.2500, Angle::Degree ),
      Angle( 45.0000, Angle::Degree ), UrsaMajor },
    { Angle( 102.0000, Angle::Degree ), Angle( 110.5000, Angle::Degree ),
      Angle( 44.5000, Angle::Degree ), Lynx },
    { Angle( 328.6250, Angle::Degree ), Angle( 329.5000, Angle::Degree ),
      Angle( 44.0000, Angle::Degree ), Cygnus },
    { Angle( 328.1250, Angle::Degree ), Angle( 328.6250, Angle::Degree ),
      Angle( 43.7500, Angle::Degree ), Cygnus },
    { Angle( 287.5000, Angle::Degree ), Angle( 291.0000, Angle::Degree ),
      Angle( 43.5000, Angle::Degree ), Cygnus },
    { Angle( 137.5000, Angle::Degree ), Angle( 152.5000, Angle::Degree ),
      Angle( 42.0000, Angle::Degree ), UrsaMajor },
    { Angle( 152.5000, Angle::Degree ), Angle( 161.7500, Angle::Degree ),
      Angle( 40.0000, Angle::Degree ), UrsaMajor },
    { Angle( 231.5000, Angle::Degree ), Angle( 236.2500, Angle::Degree ),
      Angle( 40.0000, Angle::Degree ), Bootes },
    { Angle( 236.2500, Angle::Degree ), Angle( 245.0000, Angle::Degree ),
      Angle( 40.0000, Angle::Degree ), Hercules },
    { Angle( 138.7500, Angle::Degree ), Angle( 143.7500, Angle::Degree ),
      Angle( 39.7500, Angle::Degree ), Lynx },
    { Angle(   0.0000, Angle::Degree ), Angle(  37.7500, Angle::Degree ),
      Angle( 36.7500, Angle::Degree ), Andromeda },
    { Angle(  37.7500, Angle::Degree ), Angle(  38.5000, Angle::Degree ),
      Angle( 36.7500, Angle::Degree ), Perseus },
    { Angle( 290.3750, Angle::Degree ), Angle( 291.0000, Angle::Degree ),
      Angle( 36.5000, Angle::Degree ), Lyra },
    { Angle(  67.5000, Angle::Degree ), Angle(  70.3750, Angle::Degree ),
      Angle( 36.0000, Angle::Degree ), Perseus },
    { Angle( 326.0000, Angle::Degree ), Angle( 328.1250, Angle::Degree ),
      Angle( 36.0000, Angle::Degree ), Cygnus },
    { Angle( 328.1250, Angle::Degree ), Angle( 330.0000, Angle::Degree ),
      Angle( 36.0000, Angle::Degree ), Lacerta },
    { Angle(  98.0000, Angle::Degree ), Angle( 110.5000, Angle::Degree ),
      Angle( 35.5000, Angle::Degree ), Auriga },
    { Angle( 110.5000, Angle::Degree ), Angle( 116.2500, Angle::Degree ),
      Angle( 35.5000, Angle::Degree ), Lynx },
    { Angle(   0.0000, Angle::Degree ), Angle(  30.0000, Angle::Degree ),
      Angle( 35.0000, Angle::Degree ), Andromeda },
    { Angle( 330.0000, Angle::Degree ), Angle( 342.2500, Angle::Degree ),
      Angle( 35.0000, Angle::Degree ), Lacerta },
    { Angle( 342.2500, Angle::Degree ), Angle( 343.0000, Angle::Degree ),
      Angle( 34.5000, Angle::Degree ), Lacerta },
    { Angle( 343.0000, Angle::Degree ), Angle( 352.5000, Angle::Degree ),
      Angle( 34.5000, Angle::Degree ), Andromeda },
    { Angle(  38.5000, Angle::Degree ), Angle(  40.7500, Angle::Degree ),
      Angle( 34.0000, Angle::Degree ), Perseus },
    { Angle( 161.7500, Angle::Degree ), Angle( 165.0000, Angle::Degree ),
      Angle( 34.0000, Angle::Degree ), UrsaMajor },
    { Angle( 180.0000, Angle::Degree ), Angle( 185.0000, Angle::Degree ),
      Angle( 34.0000, Angle::Degree ), CanesVenatici },
    { Angle( 116.2500, Angle::Degree ), Angle( 138.7500, Angle::Degree ),
      Angle( 33.5000, Angle::Degree ), Lynx },
    { Angle( 138.7500, Angle::Degree ), Angle( 148.2500, Angle::Degree ),
      Angle( 33.5000, Angle::Degree ), LeoMinor },
    { Angle(  10.7500, Angle::Degree ), Angle(  21.1250, Angle::Degree ),
      Angle( 33.0000, Angle::Degree ), Andromeda },
    { Angle( 227.7500, Angle::Degree ), Angle( 231.5000, Angle::Degree ),
      Angle( 33.0000, Angle::Degree ), Bootes },
    { Angle( 352.5000, Angle::Degree ), Angle( 356.2500, Angle::Degree ),
      Angle( 32.0833, Angle::Degree ), Andromeda },
    { Angle( 185.0000, Angle::Degree ), Angle( 198.7500, Angle::Degree ),
      Angle( 32.0000, Angle::Degree ), CanesVenatici },
    { Angle( 356.2500, Angle::Degree ), Angle( 360.0000, Angle::Degree ),
      Angle( 31.3333, Angle::Degree ), Andromeda }, 
    { Angle( 209.3750, Angle::Degree ), Angle( 210.5000, Angle::Degree ),
      Angle( 30.7500, Angle::Degree ), CanesVenatici },
    { Angle(  36.2500, Angle::Degree ), Angle(  40.7500, Angle::Degree ),
      Angle( 30.6667, Angle::Degree ), Triangulum },
    { Angle(  40.7500, Angle::Degree ), Angle(  67.5000, Angle::Degree ),
      Angle( 30.6667, Angle::Degree ), Perseus },
    { Angle(  67.5000, Angle::Degree ), Angle(  71.2500, Angle::Degree ),
      Angle( 30.0000, Angle::Degree ), Auriga },
    { Angle( 272.6250, Angle::Degree ), Angle( 290.3750, Angle::Degree ),
      Angle( 30.0000, Angle::Degree ), Lyra },
    { Angle( 165.0000, Angle::Degree ), Angle( 180.0000, Angle::Degree ),
      Angle( 29.0000, Angle::Degree ), UrsaMajor },
    { Angle( 295.0000, Angle::Degree ), Angle( 313.7500, Angle::Degree ),
      Angle( 29.0000, Angle::Degree ), Cygnus },
    { Angle(  71.2500, Angle::Degree ), Angle(  88.2500, Angle::Degree ),
      Angle( 28.5000, Angle::Degree ), Auriga },
    { Angle( 148.2500, Angle::Degree ), Angle( 157.5000, Angle::Degree ),
      Angle( 28.5000, Angle::Degree ), LeoMinor },
    { Angle( 198.7500, Angle::Degree ), Angle( 209.3750, Angle::Degree ),
      Angle( 28.5000, Angle::Degree ), CanesVenatici },
    { Angle(   0.0000, Angle::Degree ), Angle(   1.0000, Angle::Degree ),
      Angle( 28.0000, Angle::Degree ), Andromeda },
    { Angle(  21.1250, Angle::Degree ), Angle(  25.0000, Angle::Degree ),
      Angle( 28.0000, Angle::Degree ), Triangulum },
    { Angle(  88.2500, Angle::Degree ), Angle(  98.0000, Angle::Degree ),
      Angle( 28.0000, Angle::Degree ), Auriga },
    { Angle( 118.2500, Angle::Degree ), Angle( 120.0000, Angle::Degree ),
      Angle( 28.0000, Angle::Degree ), Gemini },
    { Angle( 313.7500, Angle::Degree ), Angle( 326.0000, Angle::Degree ),
      Angle( 28.0000, Angle::Degree ), Cygnus },
    { Angle( 288.8750, Angle::Degree ), Angle( 295.0000, Angle::Degree ),
      Angle( 27.5000, Angle::Degree ), Cygnus },
    { Angle(  28.7500, Angle::Degree ), Angle(  36.2500, Angle::Degree ),
      Angle( 27.2500, Angle::Degree ), Triangulum },
    { Angle( 242.5000, Angle::Degree ), Angle( 245.0000, Angle::Degree ),
      Angle( 27.0000, Angle::Degree ), CoronaBorealis },
    { Angle( 226.2500, Angle::Degree ), Angle( 227.7500, Angle::Degree ),
      Angle( 26.0000, Angle::Degree ), Bootes },
    { Angle( 227.7500, Angle::Degree ), Angle( 242.5000, Angle::Degree ),
      Angle( 26.0000, Angle::Degree ), CoronaBorealis },
    { Angle( 275.5000, Angle::Degree ), Angle( 283.0000, Angle::Degree ),
      Angle( 26.0000, Angle::Degree ), Lyra },
    { Angle( 161.2500, Angle::Degree ), Angle( 165.0000, Angle::Degree ),
      Angle( 25.5000, Angle::Degree ), LeoMinor },
    { Angle( 283.0000, Angle::Degree ), Angle( 288.8750, Angle::Degree ),
      Angle( 25.5000, Angle::Degree ), Lyra },
    { Angle(  25.0000, Angle::Degree ), Angle(  28.7500, Angle::Degree ),
      Angle( 25.0000, Angle::Degree ), Triangulum },
    { Angle(  10.7500, Angle::Degree ), Angle(  12.7500, Angle::Degree ),
      Angle( 23.7500, Angle::Degree ), Pisces },
    { Angle( 157.5000, Angle::Degree ), Angle( 161.2500, Angle::Degree ),
      Angle( 23.5000, Angle::Degree ), LeoMinor },
    { Angle( 318.7500, Angle::Degree ), Angle( 321.2500, Angle::Degree ),
      Angle( 23.5000, Angle::Degree ), Vulpecula },
    { Angle(  85.5000, Angle::Degree ), Angle(  88.2500, Angle::Degree ),
      Angle( 22.8333, Angle::Degree ), Taurus },
    { Angle(   1.0000, Angle::Degree ), Angle(   2.1250, Angle::Degree ),
      Angle( 22.0000, Angle::Degree ), Andromeda },
    { Angle( 238.7500, Angle::Degree ), Angle( 240.5000, Angle::Degree ),
      Angle( 22.0000, Angle::Degree ), Serpens },
    { Angle(  88.2500, Angle::Degree ), Angle(  93.2500, Angle::Degree ),
      Angle( 21.5000, Angle::Degree ), Gemini },
    { Angle( 297.5000, Angle::Degree ), Angle( 303.7500, Angle::Degree ),
      Angle( 21.2500, Angle::Degree ), Vulpecula },
    { Angle( 283.0000, Angle::Degree ), Angle( 288.7500, Angle::Degree ),
      Angle( 21.0833, Angle::Degree ), Vulpecula },
    { Angle(   2.1250, Angle::Degree ), Angle(  12.7500, Angle::Degree ),
      Angle( 21.0000, Angle::Degree ), Andromeda },
    { Angle( 303.7500, Angle::Degree ), Angle( 308.5000, Angle::Degree ),
      Angle( 20.5000, Angle::Degree ), Vulpecula },
    { Angle( 117.1250, Angle::Degree ), Angle( 118.2500, Angle::Degree ),
      Angle( 20.0000, Angle::Degree ), Gemini },
    { Angle( 308.5000, Angle::Degree ), Angle( 318.7500, Angle::Degree ),
      Angle( 19.5000, Angle::Degree ), Vulpecula },
    { Angle( 288.7500, Angle::Degree ), Angle( 297.5000, Angle::Degree ),
      Angle( 19.1667, Angle::Degree ), Vulpecula },
    { Angle(  49.2500, Angle::Degree ), Angle(  50.5000, Angle::Degree ),
      Angle( 19.0000, Angle::Degree ), Aries },
    { Angle( 283.0000, Angle::Degree ), Angle( 285.0000, Angle::Degree ),
      Angle( 18.5000, Angle::Degree ), Sagitta },
    { Angle(  85.5000, Angle::Degree ), Angle(  86.5000, Angle::Degree ),
      Angle( 18.0000, Angle::Degree ), Orion },
    { Angle(  93.2500, Angle::Degree ), Angle(  94.6250, Angle::Degree ),
      Angle( 17.5000, Angle::Degree ), Gemini },
    { Angle( 285.0000, Angle::Degree ), Angle( 297.5000, Angle::Degree ),
      Angle( 16.1667, Angle::Degree ), Sagitta },
    { Angle(  74.5000, Angle::Degree ), Angle(  80.0000, Angle::Degree ),
      Angle( 16.0000, Angle::Degree ), Taurus },
    { Angle( 238.7500, Angle::Degree ), Angle( 241.2500, Angle::Degree ),
      Angle( 16.0000, Angle::Degree ), Hercules },
    { Angle( 297.5000, Angle::Degree ), Angle( 303.7500, Angle::Degree ),
      Angle( 15.7500, Angle::Degree ), Sagitta },
    { Angle(  69.2500, Angle::Degree ), Angle(  74.5000, Angle::Degree ),
      Angle( 15.5000, Angle::Degree ), Taurus },
    { Angle(  80.0000, Angle::Degree ), Angle(  84.0000, Angle::Degree ),
      Angle( 15.5000, Angle::Degree ), Taurus },
    { Angle( 192.5000, Angle::Degree ), Angle( 202.5000, Angle::Degree ),
      Angle( 15.0000, Angle::Degree ), ComaBerenices },
    { Angle( 258.7500, Angle::Degree ), Angle( 273.7500, Angle::Degree ),
      Angle( 14.3333, Angle::Degree ), Hercules },
    { Angle( 178.0000, Angle::Degree ), Angle( 192.5000, Angle::Degree ),
      Angle( 14.0000, Angle::Degree ), ComaBerenices },
    { Angle( 112.5000, Angle::Degree ), Angle( 117.1250, Angle::Degree ),
      Angle( 13.5000, Angle::Degree ), Gemini },
    { Angle( 251.2500, Angle::Degree ), Angle( 258.7500, Angle::Degree ),
      Angle( 12.8333, Angle::Degree ), Hercules },
    { Angle(   0.0000, Angle::Degree ), Angle(   2.1250, Angle::Degree ),
      Angle( 12.5000, Angle::Degree ), Pegasus },
    { Angle(  84.0000, Angle::Degree ), Angle(  86.5000, Angle::Degree ),
      Angle( 12.5000, Angle::Degree ), Taurus },
    { Angle( 105.0000, Angle::Degree ), Angle( 112.5000, Angle::Degree ),
      Angle( 12.5000, Angle::Degree ), Gemini },
    { Angle( 316.7500, Angle::Degree ), Angle( 320.0000, Angle::Degree ),
      Angle( 12.5000, Angle::Degree ), Pegasus },
    { Angle(  94.6250, Angle::Degree ), Angle( 104.0000, Angle::Degree ),
      Angle( 12.0000, Angle::Degree ), Gemini },
    { Angle( 273.7500, Angle::Degree ), Angle( 283.0000, Angle::Degree ),
      Angle( 12.0000, Angle::Degree ), Hercules },
    { Angle( 313.1250, Angle::Degree ), Angle( 315.7500, Angle::Degree ),
      Angle( 11.8333, Angle::Degree ), Delphinus },
    { Angle( 315.7500, Angle::Degree ), Angle( 316.7500, Angle::Degree ),
      Angle( 11.8333, Angle::Degree ), Pegasus },
    { Angle( 172.7500, Angle::Degree ), Angle( 178.0000, Angle::Degree ),
      Angle( 11.0000, Angle::Degree ), Leo },
    { Angle(  93.6250, Angle::Degree ), Angle(  94.6250, Angle::Degree ),
      Angle( 10.0000, Angle::Degree ), Orion },
    { Angle( 104.0000, Angle::Degree ), Angle( 105.0000, Angle::Degree ),
      Angle( 10.0000, Angle::Degree ), Gemini },
    { Angle( 117.1250, Angle::Degree ), Angle( 118.8750, Angle::Degree ),
      Angle( 10.0000, Angle::Degree ), Cancer },
    { Angle( 357.5000, Angle::Degree ), Angle( 360.0000, Angle::Degree ),
      Angle( 10.0000, Angle::Degree ), Pegasus },
    { Angle(  25.0000, Angle::Degree ), Angle(  49.2500, Angle::Degree ),
      Angle( 9.9167, Angle::Degree ), Aries },
    { Angle( 302.1250, Angle::Degree ), Angle( 304.5000, Angle::Degree ),
      Angle( 8.5000, Angle::Degree ), Delphinus },
    { Angle( 202.5000, Angle::Degree ), Angle( 226.2500, Angle::Degree ),
      Angle( 8.0000, Angle::Degree ), Bootes },
    { Angle( 341.2500, Angle::Degree ), Angle( 357.5000, Angle::Degree ),
      Angle( 7.5000, Angle::Degree ), Pegasus },
    { Angle( 118.8750, Angle::Degree ), Angle( 138.7500, Angle::Degree ),
      Angle( 7.0000, Angle::Degree ), Cancer },
    { Angle( 138.7500, Angle::Degree ), Angle( 161.2500, Angle::Degree ),
      Angle( 7.0000, Angle::Degree ), Leo },
    { Angle( 273.7500, Angle::Degree ), Angle( 279.9333, Angle::Degree ),
      Angle( 6.2500, Angle::Degree ), Ophiuchus },
    { Angle( 279.9333, Angle::Degree ), Angle( 283.0000, Angle::Degree ),
      Angle( 6.2500, Angle::Degree ), Aquila },
    { Angle( 312.5000, Angle::Degree ), Angle( 313.1250, Angle::Degree ),
      Angle( 6.0000, Angle::Degree ), Delphinus },
    { Angle( 105.0000, Angle::Degree ), Angle( 105.2500, Angle::Degree ),
      Angle( 5.5000, Angle::Degree ), CanisMinor },
    { Angle( 273.7500, Angle::Degree ), Angle( 276.3750, Angle::Degree ),
      Angle( 4.5000, Angle::Degree ), Serpens },
    { Angle( 241.2500, Angle::Degree ), Angle( 251.2500, Angle::Degree ),
      Angle( 4.0000, Angle::Degree ), Hercules },
    { Angle( 273.7500, Angle::Degree ), Angle( 276.3750, Angle::Degree ),
      Angle( 3.0000, Angle::Degree ), Ophiuchus },
    { Angle( 322.0000, Angle::Degree ), Angle( 325.0000, Angle::Degree ),
      Angle( 2.7500, Angle::Degree ), Pegasus },
    { Angle(   0.0000, Angle::Degree ), Angle(  30.0000, Angle::Degree ),
      Angle( 2.0000, Angle::Degree ), Pisces },
    { Angle( 278.7500, Angle::Degree ), Angle( 283.0000, Angle::Degree ),
      Angle( 2.0000, Angle::Degree ), Serpens },
    { Angle( 304.5000, Angle::Degree ), Angle( 312.5000, Angle::Degree ),
      Angle( 2.0000, Angle::Degree ), Delphinus },
    { Angle( 312.5000, Angle::Degree ), Angle( 320.0000, Angle::Degree ),
      Angle( 2.0000, Angle::Degree ), Equuleus },
    { Angle( 320.0000, Angle::Degree ), Angle( 322.0000, Angle::Degree ),
      Angle( 2.0000, Angle::Degree ), Pegasus },
    { Angle( 330.0000, Angle::Degree ), Angle( 341.2500, Angle::Degree ),
      Angle( 2.0000, Angle::Degree ), Pegasus },
    { Angle( 325.0000, Angle::Degree ), Angle( 330.0000, Angle::Degree ),
      Angle( 1.7500, Angle::Degree ), Pegasus },
    { Angle( 105.2500, Angle::Degree ), Angle( 108.0000, Angle::Degree ),
      Angle( 1.5000, Angle::Degree ), CanisMinor },
    { Angle(  53.7500, Angle::Degree ), Angle(  69.2500, Angle::Degree ),
      Angle( 0.0000, Angle::Degree ), Taurus },
    { Angle(  69.2500, Angle::Degree ), Angle(  70.0000, Angle::Degree ),
      Angle( 0.0000, Angle::Degree ), Orion },
    { Angle( 108.0000, Angle::Degree ), Angle( 121.2500, Angle::Degree ),
      Angle( 0.0000, Angle::Degree ), CanisMinor },
    { Angle( 220.0000, Angle::Degree ), Angle( 226.2500, Angle::Degree ),
      Angle( 0.0000, Angle::Degree ), Virgo },
    { Angle( 267.5000, Angle::Degree ), Angle( 273.7500, Angle::Degree ),
      Angle( 0.0000, Angle::Degree ), Ophiuchus },
    { Angle(  39.7500, Angle::Degree ), Angle(  49.2500, Angle::Degree ),
      Angle( -1.7500, Angle::Degree ), Cetus },
    { Angle(  49.2500, Angle::Degree ), Angle(  53.7500, Angle::Degree ),
      Angle( -1.7500, Angle::Degree ), Taurus },
    { Angle( 226.2500, Angle::Degree ), Angle( 244.0000, Angle::Degree ),
      Angle( -3.2500, Angle::Degree ), Serpens },
    { Angle(  70.0000, Angle::Degree ), Angle(  76.2500, Angle::Degree ),
      Angle( -4.0000, Angle::Degree ), Orion },
    { Angle(  87.5000, Angle::Degree ), Angle(  93.6250, Angle::Degree ),
      Angle( -4.0000, Angle::Degree ), Orion },
    { Angle( 267.5000, Angle::Degree ), Angle( 269.5000, Angle::Degree ),
      Angle( -4.0000, Angle::Degree ), Serpens },
    { Angle( 273.7500, Angle::Degree ), Angle( 278.7500, Angle::Degree ),
      Angle( -4.0000, Angle::Degree ), Serpens },
    { Angle( 278.7500, Angle::Degree ), Angle( 283.0000, Angle::Degree ),
      Angle( -4.0000, Angle::Degree ), Aquila },
    { Angle( 341.2500, Angle::Degree ), Angle( 357.5000, Angle::Degree ),
      Angle( -4.0000, Angle::Degree ), Pisces },
    { Angle( 161.2500, Angle::Degree ), Angle( 172.7500, Angle::Degree ),
      Angle( -6.0000, Angle::Degree ), Leo },
    { Angle( 172.7500, Angle::Degree ), Angle( 177.5000, Angle::Degree ),
      Angle( -6.0000, Angle::Degree ), Virgo },
    { Angle(   0.0000, Angle::Degree ), Angle(   5.0000, Angle::Degree ),
      Angle( -7.0000, Angle::Degree ), Pisces },
    { Angle( 357.5000, Angle::Degree ), Angle( 360.0000, Angle::Degree ),
      Angle( -7.0000, Angle::Degree ), Pisces },
    { Angle( 213.7500, Angle::Degree ), Angle( 220.0000, Angle::Degree ),
      Angle( -8.0000, Angle::Degree ), Virgo },
    { Angle( 238.7500, Angle::Degree ), Angle( 244.0000, Angle::Degree ),
      Angle( -8.0000, Angle::Degree ), Ophiuchus },
    { Angle( 300.0000, Angle::Degree ), Angle( 308.0000, Angle::Degree ),
      Angle( -9.0000, Angle::Degree ), Aquila },
    { Angle( 320.0000, Angle::Degree ), Angle( 328.0000, Angle::Degree ),
      Angle( -9.0000, Angle::Degree ), Aquarius },
    { Angle( 257.5000, Angle::Degree ), Angle( 269.5000, Angle::Degree ),
      Angle( -10.0000, Angle::Degree ), Ophiuchus },
    { Angle(  87.5000, Angle::Degree ), Angle( 121.2500, Angle::Degree ),
      Angle( -11.0000, Angle::Degree ), Monoceros },
    { Angle(  73.7500, Angle::Degree ), Angle(  76.2500, Angle::Degree ),
      Angle( -11.0000, Angle::Degree ), Eridanus },
    { Angle(  76.2500, Angle::Degree ), Angle(  87.5000, Angle::Degree ),
      Angle( -11.0000, Angle::Degree ), Orion },
    { Angle( 121.2500, Angle::Degree ), Angle( 125.5000, Angle::Degree ),
      Angle( -11.0000, Angle::Degree ), Hydra },
    { Angle( 143.7500, Angle::Degree ), Angle( 161.2500, Angle::Degree ),
      Angle( -11.0000, Angle::Degree ), Sextans },
    { Angle( 177.5000, Angle::Degree ), Angle( 192.5000, Angle::Degree ),
      Angle( -11.0000, Angle::Degree ), Virgo },
    { Angle( 263.7500, Angle::Degree ), Angle( 265.0000, Angle::Degree ),
      Angle( -11.6667, Angle::Degree ), Ophiuchus },
    { Angle( 283.0000, Angle::Degree ), Angle( 300.0000, Angle::Degree ),
      Angle( -12.0333, Angle::Degree ), Aquila },
    { Angle(  72.5000, Angle::Degree ), Angle(  73.7500, Angle::Degree ),
      Angle( -14.5000, Angle::Degree ), Eridanus },
    { Angle( 308.0000, Angle::Degree ), Angle( 320.0000, Angle::Degree ),
      Angle( -15.0000, Angle::Degree ), Aquarius },
    { Angle( 257.5000, Angle::Degree ), Angle( 273.7500, Angle::Degree ),
      Angle( -16.0000, Angle::Degree ), Serpens },
    { Angle( 273.7500, Angle::Degree ), Angle( 283.0000, Angle::Degree ),
      Angle( -16.0000, Angle::Degree ), Scutum },
    { Angle( 125.5000, Angle::Degree ), Angle( 128.7500, Angle::Degree ),
      Angle( -17.0000, Angle::Degree ), Hydra },
    { Angle( 244.0000, Angle::Degree ), Angle( 245.6250, Angle::Degree ),
      Angle( -18.2500, Angle::Degree ), Ophiuchus },
    { Angle( 128.7500, Angle::Degree ), Angle( 136.2500, Angle::Degree ),
      Angle( -19.0000, Angle::Degree ), Hydra },
    { Angle( 161.2500, Angle::Degree ), Angle( 162.5000, Angle::Degree ),
      Angle( -19.0000, Angle::Degree ), Crater },
    { Angle( 244.0000, Angle::Degree ), Angle( 245.6250, Angle::Degree ),
      Angle( -19.2500, Angle::Degree ), Scorpius },
    { Angle( 235.0000, Angle::Degree ), Angle( 238.7500, Angle::Degree ),
      Angle( -20.0000, Angle::Degree ), Libra },
    { Angle( 188.7500, Angle::Degree ), Angle( 192.5000, Angle::Degree ),
      Angle( -22.0000, Angle::Degree ), Corvus },
    { Angle( 192.5000, Angle::Degree ), Angle( 213.7500, Angle::Degree ),
      Angle( -22.0000, Angle::Degree ), Virgo },
    { Angle( 136.2500, Angle::Degree ), Angle( 146.2500, Angle::Degree ),
      Angle( -24.0000, Angle::Degree ), Hydra },
    { Angle(  25.0000, Angle::Degree ), Angle(  39.7500, Angle::Degree ),
      Angle( -24.3833, Angle::Degree ), Cetus },
    { Angle(  39.7500, Angle::Degree ), Angle(  56.2500, Angle::Degree ),
      Angle( -24.3833, Angle::Degree ), Eridanus },
    { Angle( 162.5000, Angle::Degree ), Angle( 177.5000, Angle::Degree ),
      Angle( -24.5000, Angle::Degree ), Crater },
    { Angle( 177.5000, Angle::Degree ), Angle( 188.7500, Angle::Degree ),
      Angle( -24.5000, Angle::Degree ), Corvus },
    { Angle( 213.7500, Angle::Degree ), Angle( 223.7500, Angle::Degree ),
      Angle( -24.5000, Angle::Degree ), Libra },
    { Angle( 244.0000, Angle::Degree ), Angle( 251.2500, Angle::Degree ),
      Angle( -24.5833, Angle::Degree ), Ophiuchus },
    { Angle(   0.0000, Angle::Degree ), Angle(  25.0000, Angle::Degree ),
      Angle( -25.5000, Angle::Degree ), Cetus },
    { Angle( 320.0000, Angle::Degree ), Angle( 328.0000, Angle::Degree ),
      Angle( -25.5000, Angle::Degree ), Capricornus },
    { Angle( 328.0000, Angle::Degree ), Angle( 357.5000, Angle::Degree ),
      Angle( -25.5000, Angle::Degree ), Aquarius },
    { Angle( 357.5000, Angle::Degree ), Angle( 360.0000, Angle::Degree ),
      Angle( -25.5000, Angle::Degree ), Cetus },
    { Angle( 146.2500, Angle::Degree ), Angle( 153.7500, Angle::Degree ),
      Angle( -26.5000, Angle::Degree ), Hydra },
    { Angle(  70.5000, Angle::Degree ), Angle(  72.5000, Angle::Degree ),
      Angle( -27.2500, Angle::Degree ), Eridanus },
    { Angle(  72.5000, Angle::Degree ), Angle(  91.7500, Angle::Degree ),
      Angle( -27.2500, Angle::Degree ), Lepus },
    { Angle( 300.0000, Angle::Degree ), Angle( 320.0000, Angle::Degree ),
      Angle( -28.0000, Angle::Degree ), Capricornus },
    { Angle( 153.7500, Angle::Degree ), Angle( 158.7500, Angle::Degree ),
      Angle( -29.1667, Angle::Degree ), Hydra },
    { Angle( 188.7500, Angle::Degree ), Angle( 223.7500, Angle::Degree ),
      Angle( -29.5000, Angle::Degree ), Hydra },
    { Angle( 223.7500, Angle::Degree ), Angle( 235.0000, Angle::Degree ),
      Angle( -29.5000, Angle::Degree ), Libra },
    { Angle( 235.0000, Angle::Degree ), Angle( 240.0000, Angle::Degree ),
      Angle( -29.5000, Angle::Degree ), Scorpius },
    { Angle(  68.7500, Angle::Degree ), Angle(  70.5000, Angle::Degree ),
      Angle( -30.0000, Angle::Degree ), Eridanus },
    { Angle( 251.2500, Angle::Degree ), Angle( 264.0000, Angle::Degree ),
      Angle( -30.0000, Angle::Degree ), Ophiuchus },
    { Angle( 264.0000, Angle::Degree ), Angle( 267.5000, Angle::Degree ),
      Angle( -30.0000, Angle::Degree ), Sagittarius },
    { Angle( 158.7500, Angle::Degree ), Angle( 162.5000, Angle::Degree ),
      Angle( -31.1667, Angle::Degree ), Hydra },
    { Angle(  91.7500, Angle::Degree ), Angle( 110.5000, Angle::Degree ),
      Angle( -33.0000, Angle::Degree ), CanisMajor },
    { Angle( 183.7500, Angle::Degree ), Angle( 188.7500, Angle::Degree ),
      Angle( -33.0000, Angle::Degree ), Hydra },
    { Angle( 162.5000, Angle::Degree ), Angle( 183.7500, Angle::Degree ),
      Angle( -35.0000, Angle::Degree ), Hydra },
    { Angle(  52.5000, Angle::Degree ), Angle(  56.2500, Angle::Degree ),
      Angle( -36.0000, Angle::Degree ), Fornax },
    { Angle( 125.5000, Angle::Degree ), Angle( 140.5000, Angle::Degree ),
      Angle( -36.7500, Angle::Degree ), Pyxis },
    { Angle(  64.0000, Angle::Degree ), Angle(  68.7500, Angle::Degree ),
      Angle( -37.0000, Angle::Degree ), Eridanus },
    { Angle( 267.5000, Angle::Degree ), Angle( 287.5000, Angle::Degree ),
      Angle( -37.0000, Angle::Degree ), Sagittarius },
    { Angle( 320.0000, Angle::Degree ), Angle( 345.0000, Angle::Degree ),
      Angle( -37.0000, Angle::Degree ), PiscisAustrinus },
    { Angle( 345.0000, Angle::Degree ), Angle( 350.0000, Angle::Degree ),
      Angle( -37.0000, Angle::Degree ), Sculptor },
    { Angle(  45.0000, Angle::Degree ), Angle(  52.5000, Angle::Degree ),
      Angle( -39.5833, Angle::Degree ), Fornax },
    { Angle( 140.5000, Angle::Degree ), Angle( 165.0000, Angle::Degree ),
      Angle( -39.7500, Angle::Degree ), Antlia },
    { Angle(   0.0000, Angle::Degree ), Angle(  25.0000, Angle::Degree ),
      Angle( -40.0000, Angle::Degree ), Sculptor },
    { Angle(  25.0000, Angle::Degree ), Angle(  45.0000, Angle::Degree ),
      Angle( -40.0000, Angle::Degree ), Fornax },
    { Angle(  58.0000, Angle::Degree ), Angle(  64.0000, Angle::Degree ),
      Angle( -40.0000, Angle::Degree ), Eridanus },
    { Angle( 350.0000, Angle::Degree ), Angle( 360.0000, Angle::Degree ),
      Angle( -40.0000, Angle::Degree ), Sculptor },
    { Angle( 212.5000, Angle::Degree ), Angle( 223.7500, Angle::Degree ),
      Angle( -42.0000, Angle::Degree ), Centaurus },
    { Angle( 235.0000, Angle::Degree ), Angle( 240.0000, Angle::Degree ),
      Angle( -42.0000, Angle::Degree ), Lupus },
    { Angle( 240.0000, Angle::Degree ), Angle( 246.3125, Angle::Degree ),
      Angle( -42.0000, Angle::Degree ), Scorpius },
    { Angle(  72.5000, Angle::Degree ), Angle(  75.0000, Angle::Degree ),
      Angle( -43.0000, Angle::Degree ), Caelum },
    { Angle(  75.0000, Angle::Degree ), Angle(  98.7500, Angle::Degree ),
      Angle( -43.0000, Angle::Degree ), Columba },
    { Angle( 120.0000, Angle::Degree ), Angle( 125.5000, Angle::Degree ),
      Angle( -43.0000, Angle::Degree ), Puppis },
    { Angle(  51.2500, Angle::Degree ), Angle(  58.0000, Angle::Degree ),
      Angle( -44.0000, Angle::Degree ), Eridanus },
    { Angle( 246.3125, Angle::Degree ), Angle( 267.5000, Angle::Degree ),
      Angle( -45.5000, Angle::Degree ), Scorpius },
    { Angle( 267.5000, Angle::Degree ), Angle( 287.5000, Angle::Degree ),
      Angle( -45.5000, Angle::Degree ), CoronaAustralis },
    { Angle( 287.5000, Angle::Degree ), Angle( 305.0000, Angle::Degree ),
      Angle( -45.5000, Angle::Degree ), Sagittarius },
    { Angle( 305.0000, Angle::Degree ), Angle( 320.0000, Angle::Degree ),
      Angle( -45.5000, Angle::Degree ), Microscopium },
    { Angle(  45.0000, Angle::Degree ), Angle(  51.2500, Angle::Degree ),
      Angle( -46.0000, Angle::Degree ), Eridanus },
    { Angle(  67.5000, Angle::Degree ), Angle(  72.5000, Angle::Degree ),
      Angle( -46.5000, Angle::Degree ), Caelum },
    { Angle( 230.0000, Angle::Degree ), Angle( 235.0000, Angle::Degree ),
      Angle( -48.0000, Angle::Degree ), Lupus },
    { Angle(   0.0000, Angle::Degree ), Angle(  35.0000, Angle::Degree ),
      Angle( -48.1667, Angle::Degree ), Phoenix },
    { Angle(  40.0000, Angle::Degree ), Angle(  45.0000, Angle::Degree ),
      Angle( -49.0000, Angle::Degree ), Eridanus },
    { Angle(  61.2500, Angle::Degree ), Angle(  64.0000, Angle::Degree ),
      Angle( -49.0000, Angle::Degree ), Horologium },
    { Angle(  64.0000, Angle::Degree ), Angle(  67.5000, Angle::Degree ),
      Angle( -49.0000, Angle::Degree ), Caelum },
    { Angle( 320.0000, Angle::Degree ), Angle( 330.0000, Angle::Degree ),
      Angle( -50.0000, Angle::Degree ), Grus },
    { Angle(  90.0000, Angle::Degree ), Angle( 120.0000, Angle::Degree ),
      Angle( -50.7500, Angle::Degree ), Puppis },
    { Angle( 120.0000, Angle::Degree ), Angle( 122.5000, Angle::Degree ),
      Angle( -50.7500, Angle::Degree ), Vela },
    { Angle(  36.2500, Angle::Degree ), Angle(  40.0000, Angle::Degree ),
      Angle( -51.0000, Angle::Degree ), Eridanus },
    { Angle(  57.5000, Angle::Degree ), Angle(  61.2500, Angle::Degree ),
      Angle( -51.0000, Angle::Degree ), Horologium },
    { Angle(   0.0000, Angle::Degree ), Angle(  27.5000, Angle::Degree ),
      Angle( -51.5000, Angle::Degree ), Phoenix },
    { Angle(  90.0000, Angle::Degree ), Angle(  92.5000, Angle::Degree ),
      Angle( -52.5000, Angle::Degree ), Carina },
    { Angle( 122.5000, Angle::Degree ), Angle( 126.7500, Angle::Degree ),
      Angle( -53.0000, Angle::Degree ), Vela },
    { Angle(  52.5000, Angle::Degree ), Angle(  57.5000, Angle::Degree ),
      Angle( -53.1667, Angle::Degree ), Horologium },
    { Angle(  57.5000, Angle::Degree ), Angle(  60.0000, Angle::Degree ),
      Angle( -53.1667, Angle::Degree ), Dorado },
    { Angle(   0.0000, Angle::Degree ), Angle(  23.7500, Angle::Degree ),
      Angle( -53.5000, Angle::Degree ), Phoenix },
    { Angle(  32.5000, Angle::Degree ), Angle(  36.2500, Angle::Degree ),
      Angle( -54.0000, Angle::Degree ), Eridanus },
    { Angle(  67.5000, Angle::Degree ), Angle(  75.0000, Angle::Degree ),
      Angle( -54.0000, Angle::Degree ), Pictor },
    { Angle( 225.7500, Angle::Degree ), Angle( 230.0000, Angle::Degree ),
      Angle( -54.0000, Angle::Degree ), Lupus },
    { Angle( 126.7500, Angle::Degree ), Angle( 132.5000, Angle::Degree ),
      Angle( -54.5000, Angle::Degree ), Vela },
    { Angle(  92.5000, Angle::Degree ), Angle(  97.5000, Angle::Degree ),
      Angle( -55.0000, Angle::Degree ), Carina },
    { Angle( 177.5000, Angle::Degree ), Angle( 192.5000, Angle::Degree ),
      Angle( -55.0000, Angle::Degree ), Centaurus },
    { Angle( 212.5000, Angle::Degree ), Angle( 225.7500, Angle::Degree ),
      Angle( -55.0000, Angle::Degree ), Lupus },
    { Angle( 225.7500, Angle::Degree ), Angle( 230.0000, Angle::Degree ),
      Angle( -55.0000, Angle::Degree ), Norma },
    { Angle(  60.0000, Angle::Degree ), Angle(  65.0000, Angle::Degree ),
      Angle( -56.5000, Angle::Degree ), Dorado },
    { Angle( 132.5000, Angle::Degree ), Angle( 165.0000, Angle::Degree ),
      Angle( -56.5000, Angle::Degree ), Vela },
    { Angle( 165.0000, Angle::Degree ), Angle( 168.7500, Angle::Degree ),
      Angle( -56.5000, Angle::Degree ), Centaurus },
    { Angle( 262.5000, Angle::Degree ), Angle( 270.0000, Angle::Degree ),
      Angle( -57.0000, Angle::Degree ), Ara },
    { Angle( 270.0000, Angle::Degree ), Angle( 305.0000, Angle::Degree ),
      Angle( -57.0000, Angle::Degree ), Telescopium },
    { Angle( 330.0000, Angle::Degree ), Angle( 350.0000, Angle::Degree ),
      Angle( -57.0000, Angle::Degree ), Grus },
    { Angle(  48.0000, Angle::Degree ), Angle(  52.5000, Angle::Degree ),
      Angle( -57.5000, Angle::Degree ), Horologium },
    { Angle(  75.0000, Angle::Degree ), Angle(  82.5000, Angle::Degree ),
      Angle( -57.5000, Angle::Degree ), Pictor },
    { Angle(  97.5000, Angle::Degree ), Angle( 102.5000, Angle::Degree ),
      Angle( -58.0000, Angle::Degree ), Carina },
    { Angle(   0.0000, Angle::Degree ), Angle(  20.0000, Angle::Degree ),
      Angle( -58.5000, Angle::Degree ), Phoenix },
    { Angle(  20.0000, Angle::Degree ), Angle(  32.5000, Angle::Degree ),
      Angle( -58.5000, Angle::Degree ), Eridanus },
    { Angle( 350.0000, Angle::Degree ), Angle( 360.0000, Angle::Degree ),
      Angle( -58.5000, Angle::Degree ), Phoenix },
    { Angle(  65.0000, Angle::Degree ), Angle(  68.7500, Angle::Degree ),
      Angle( -59.0000, Angle::Degree ), Dorado },
    { Angle( 230.0000, Angle::Degree ), Angle( 246.3125, Angle::Degree ),
      Angle( -60.0000, Angle::Degree ), Norma },
    { Angle( 305.0000, Angle::Degree ), Angle( 320.0000, Angle::Degree ),
      Angle( -60.0000, Angle::Degree ), Indus },
    { Angle(  82.5000, Angle::Degree ), Angle(  90.0000, Angle::Degree ),
      Angle( -61.0000, Angle::Degree ), Pictor },
    { Angle( 227.5000, Angle::Degree ), Angle( 230.0000, Angle::Degree ),
      Angle( -61.0000, Angle::Degree ), Circinus },
    { Angle( 246.3125, Angle::Degree ), Angle( 248.7500, Angle::Degree ),
      Angle( -61.0000, Angle::Degree ), Ara },
    { Angle( 223.7500, Angle::Degree ), Angle( 227.5000, Angle::Degree ),
      Angle( -63.5833, Angle::Degree ), Circinus },
    { Angle( 248.7500, Angle::Degree ), Angle( 251.2500, Angle::Degree ),
      Angle( -63.5833, Angle::Degree ), Ara },
    { Angle(  90.0000, Angle::Degree ), Angle( 102.5000, Angle::Degree ),
      Angle( -64.0000, Angle::Degree ), Pictor },
    { Angle( 102.5000, Angle::Degree ), Angle( 135.5000, Angle::Degree ),
      Angle( -64.0000, Angle::Degree ), Carina },
    { Angle( 168.7500, Angle::Degree ), Angle( 177.5000, Angle::Degree ),
      Angle( -64.0000, Angle::Degree ), Centaurus },
    { Angle( 177.5000, Angle::Degree ), Angle( 192.5000, Angle::Degree ),
      Angle( -64.0000, Angle::Degree ), Crux },
    { Angle( 192.5000, Angle::Degree ), Angle( 218.0000, Angle::Degree ),
      Angle( -64.0000, Angle::Degree ), Centaurus },
    { Angle( 202.5000, Angle::Degree ), Angle( 205.0000, Angle::Degree ),
      Angle( -65.0000, Angle::Degree ), Circinus },
    { Angle( 251.2500, Angle::Degree ), Angle( 252.5000, Angle::Degree ),
      Angle( -65.0000, Angle::Degree ), Ara },
    { Angle(  32.5000, Angle::Degree ), Angle(  48.0000, Angle::Degree ),
      Angle( -67.5000, Angle::Degree ), Horologium },
    { Angle(  48.0000, Angle::Degree ), Angle(  68.7500, Angle::Degree ),
      Angle( -67.5000, Angle::Degree ), Reticulum },
    { Angle( 221.2500, Angle::Degree ), Angle( 223.7500, Angle::Degree ),
      Angle( -67.5000, Angle::Degree ), Circinus },
    { Angle( 252.5000, Angle::Degree ), Angle( 262.5000, Angle::Degree ),
      Angle( -67.5000, Angle::Degree ), Ara },
    { Angle( 262.5000, Angle::Degree ), Angle( 270.0000, Angle::Degree ),
      Angle( -67.5000, Angle::Degree ), Pavo },
    { Angle( 330.0000, Angle::Degree ), Angle( 350.0000, Angle::Degree ),
      Angle( -67.5000, Angle::Degree ), Tucana },
    { Angle(  68.7500, Angle::Degree ), Angle(  98.7500, Angle::Degree ),
      Angle( -70.0000, Angle::Degree ), Dorado },
    { Angle( 205.0000, Angle::Degree ), Angle( 221.2500, Angle::Degree ),
      Angle( -70.0000, Angle::Degree ), Circinus },
    { Angle( 221.2500, Angle::Degree ), Angle( 255.0000, Angle::Degree ),
      Angle( -70.0000, Angle::Degree ), TriangulumAustrale },
    { Angle(   0.0000, Angle::Degree ), Angle(  20.0000, Angle::Degree ),
      Angle( -75.0000, Angle::Degree ), Tucana },
    { Angle(  52.5000, Angle::Degree ), Angle(  68.7500, Angle::Degree ),
      Angle( -75.0000, Angle::Degree ), Hydrus },
    { Angle(  98.7500, Angle::Degree ), Angle( 135.5000, Angle::Degree ),
      Angle( -75.0000, Angle::Degree ), Volans },
    { Angle( 135.5000, Angle::Degree ), Angle( 168.7500, Angle::Degree ),
      Angle( -75.0000, Angle::Degree ), Carina },
    { Angle( 168.7500, Angle::Degree ), Angle( 205.0000, Angle::Degree ),
      Angle( -75.0000, Angle::Degree ), Musca },
    { Angle( 270.0000, Angle::Degree ), Angle( 320.0000, Angle::Degree ),
      Angle( -75.0000, Angle::Degree ), Pavo },
    { Angle( 320.0000, Angle::Degree ), Angle( 350.0000, Angle::Degree ),
      Angle( -75.0000, Angle::Degree ), Indus },
    { Angle( 350.0000, Angle::Degree ), Angle( 360.0000, Angle::Degree ),
      Angle( -75.0000, Angle::Degree ), Tucana },
    { Angle(  11.2500, Angle::Degree ), Angle(  20.0000, Angle::Degree ),
      Angle( -76.0000, Angle::Degree ), Tucana },
    { Angle(   0.0000, Angle::Degree ), Angle(  52.5000, Angle::Degree ),
      Angle( -82.5000, Angle::Degree ), Hydrus },
    { Angle( 115.0000, Angle::Degree ), Angle( 205.0000, Angle::Degree ),
      Angle( -82.5000, Angle::Degree ), Chamaeleon },
    { Angle( 205.0000, Angle::Degree ), Angle( 270.0000, Angle::Degree ),
      Angle( -82.5000, Angle::Degree ), Apus },
    { Angle(  52.5000, Angle::Degree ), Angle( 115.0000, Angle::Degree ),
      Angle( -85.0000, Angle::Degree ), Mensa },
    { Angle(   0.0000, Angle::Degree ), Angle( 360.0000, Angle::Degree ),
      Angle( -90.0000, Angle::Degree ), Octans }
};


//-----------------------------------------------------------------------------
}                                                                   //namespace


//*****************************************************************************


const ConstellationInfo &
GetConstellationInfo( Constellation constellation )
{
    Assert( constellation < NumConstellations );
    return s_constellationInfo[ constellation ];
}

//=============================================================================

Constellation
GetConstellation( Equatorial equatorialPos, double epoch )
{
    Equatorial equatorial1875;
    if ( epoch == B1875 )
        equatorial1875 = equatorialPos;
    else
        equatorial1875 = Precession( B1875, epoch ).Reduce( equatorialPos );
    Angle ra = equatorial1875.RightAscension();
    Angle dec = equatorial1875.Declination();
    for ( int i = 0; i < ARRAY_LENGTH( s_boundaryData ); ++i )
    {
        BoundaryDatum & bndry = s_boundaryData[ i ];
        if ( (bndry.decLower <= dec)
             && (bndry.raLower <= ra) && (ra < bndry.raUpper) )
            return bndry.constellation;
    }
    Assert( 0 && "No constellation found" );
    return NumConstellations;
}

//=============================================================================

#ifdef DEBUG


namespace
{                                                                   //namespace
//-----------------------------------------------------------------------------

struct TestDatum
{
    double raH;
    double decD;
    double epoch;
    string constAbbrev;
};

TestDatum s_testData[]
= {
    {  9.0000,  65.0000, B1950, "UMa" },
    { 23.5000, -20.0000, B1950, "Aqr" },
    {  5.1200,   9.1200, B1950, "Ori" },
    {  9.4555, -19.9000, B1950, "Hya" },
    { 12.8888,  22.0000, B1950, "Com" },
    { 15.6687, -12.1234, B1950, "Lib" },
    { 19.0000, -40.0000, B1950, "CrA" },
    {  6.2222, -81.1234, B1950, "Men" }
};

//-----------------------------------------------------------------------------
}                                                                   //namespace

//-----------------------------------------------------------------------------

bool
TestConstellations( )
{
    bool ok = true;
    cout << "Testing Constellations" << endl;

    for ( int i = 0; i < ARRAY_LENGTH( s_testData ); ++i )
    {
        TestDatum & datum = s_testData[ i ];
        cout << "Equatorial( " << datum.raH << ", " << datum.decD << " )"
             << endl;
        Equatorial equat( Angle( AngleHMS( datum.raH ) ),
                          Angle( datum.decD, Angle::Degree ) );
        cout << "GetConstellation( equat, " << datum.epoch << " )" << endl;
        Constellation constellation = GetConstellation( equat, datum.epoch );
        cout << "GetConstellationInfo( " << constellation << " )" << endl;
        const ConstellationInfo & constInfo
                = GetConstellationInfo( constellation );
        TESTCHECK( constInfo.Abbrev(), datum.constAbbrev, &ok );
    }
    
    if ( ok )
        cout << "Constellations PASSED." << endl << endl;
    else
        cout << "Constellations FAILED." << endl << endl;
    return ok;
}


#endif


//*****************************************************************************

}                                                      //namespace EpsilonDelta
